@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
       None
@@ -27,6 +28,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
    Modifications:
 */
 
+#define FFEEQUIV_DEBUG 0
+
 /* Include files. */
 
 #include "proj.h"
@@ -111,7 +114,12 @@ ffeequiv_layout_local_ (ffeequiv eq)
   bool init;
 
   assert (eq != NULL);
-  assert (ffeequiv_common (eq) == NULL);
+
+  if (ffeequiv_common (eq) != NULL)
+    {				/* Put in common due to programmer error. */
+      ffeequiv_kill (eq);
+      return;
+    }
 
   /* First find the symbol which, in the list of lists, has the reference
      with the greatest offset, which means that symbol is the root symbol (it
@@ -121,11 +129,19 @@ ffeequiv_layout_local_ (ffeequiv eq)
   sr = NULL;			/* No sym found yet. */
   ok = TRUE;
 
+#if FFEEQUIV_DEBUG
+  fprintf (stderr, "Equiv1:\n");
+#endif
+
   for (list = ffeequiv_list (eq);
        list != NULL;
        list = ffebld_trail (list))
     {				/* For every equivalence list in the list of
 				   equivs */
+#if FFEEQUIV_DEBUG
+      fprintf (stderr, "(");
+#endif
+
       for (item = ffebld_head (list);
 	   item != NULL;
 	   item = ffebld_trail (item))
@@ -134,6 +150,11 @@ ffeequiv_layout_local_ (ffeequiv eq)
 	  s = ffeequiv_symbol (eqv);
 	  if (s == NULL)
 	    continue;		/* Ignore me. */
+
+#if FFEEQUIV_DEBUG
+	  fprintf (stderr, "%s,", ffesymbol_text (s));
+#endif
+
 	  assert (ffesymbol_storage (s) == NULL);	/* No storage yet. */
 	  ffesymbol_set_equiv (s, NULL);	/* Equiv area slated for
 						   death. */
@@ -145,6 +166,10 @@ ffeequiv_layout_local_ (ffeequiv eq)
 	      sr = s;
 	    }
 	}
+#if FFEEQUIV_DEBUG
+      fprintf (stderr, ")\n");
+#endif
+
     }
 
   if (!ok)
@@ -216,8 +241,16 @@ ffeequiv_layout_local_ (ffeequiv eq)
      do the actual storage allocation.	Keep doing this until we've gone
      through them all without making any new storage objects. */
 
+#if FFEEQUIV_DEBUG
+  fprintf (stderr, "Equiv2:\n");
+#endif
+
   do
     {
+#if FFEEQUIV_DEBUG
+      fprintf (stderr, "  Equiv3:\n");
+#endif
+
       new_storage = FALSE;
       need_storage = FALSE;
       for (list = ffeequiv_list (eq);
@@ -225,6 +258,10 @@ ffeequiv_layout_local_ (ffeequiv eq)
 	   list = ffebld_trail (list))
 	{			/* For every equivalence list in the list of
 				   equivs */
+#if FFEEQUIV_DEBUG
+	  fprintf (stderr, "  (");
+#endif
+
 	  root_offset = 0;
 	  sr = NULL;
 	  root = NULL;
@@ -233,13 +270,19 @@ ffeequiv_layout_local_ (ffeequiv eq)
 	       item = ffebld_trail (item))
 	    {			/* For every equivalence item in the list */
 	      var = ffebld_head (item);
-	      need_storage = TRUE;	/* Somebody is likely to need
-					   storage. */
 	      sv = ffeequiv_symbol (var);
 	      if (sv == NULL)
 		continue;	/* Ignore me. */
+
+#if FFEEQUIV_DEBUG
+	      fprintf (stderr, "%s,", ffesymbol_text (sv));
+#endif
+
+	      need_storage = TRUE;	/* Somebody is likely to need
+					   storage. */
 	      if ((vst = ffesymbol_storage (sv)) == NULL)
 		continue;	/* No storage for this guy, try another. */
+
 	      ffeequiv_offset_ (&var_offset, sv, var, FALSE,
 				ffestorag_offset (vst));
 	      if ((var_offset > root_offset) || (sr == NULL))
@@ -251,9 +294,16 @@ ffeequiv_layout_local_ (ffeequiv eq)
 		}
 	    }
 	  if (sr == NULL)	/* No storage to go on, try later. */
-	    continue;
+	    {
+#if FFEEQUIV_DEBUG
+	      fprintf (stderr, ")\n");
+#endif
+	      continue;
+	    }
 
-	  need_storage = FALSE;	/* Everyone in this sublist will get storage! */
+#if FFEEQUIV_DEBUG
+	  fprintf (stderr, ") %s:\n  (", ffesymbol_text (sr));
+#endif
 
 	  /* We now know the root symbol/expr and the operating offset of
 	     that root into the equivalence area.  The other expressions in
@@ -265,16 +315,26 @@ ffeequiv_layout_local_ (ffeequiv eq)
 	       item = ffebld_trail (item))
 	    {			/* For every equivalence item in the list */
 	      var = ffebld_head (item);
-	      if (var == root)
-		continue;	/* Except root, of course. */
 	      sv = ffeequiv_symbol (var);
 	      if (sv == NULL)
 		continue;	/* Except erroneous stuff (opANY). */
-	      ffesymbol_set_equiv (sv, NULL);	/* Don't need this ref
-						   anymore. */
+	      if (var == root)
+		{
+		  /* The last root symbol we see must therefore
+		     (by static deduction) be the first-listed "rooted" item
+		     in the EQUIVALENCE statements pertaining to this area.  */
+		  ffestorag_set_symbol (st, sv);
+		  continue;	/* Root sym already set up. */
+		}
+
 	      if (!ffeequiv_offset_ (&var_offset, sv, var, TRUE, root_offset))
 		continue;	/* Attempt to start sym prior to equiv area! */
 
+#if FFEEQUIV_DEBUG
+	      fprintf (stderr, "%s:%ld,", ffesymbol_text (sv),
+		       (long) var_offset);
+#endif
+
 	      if (ffesymbol_rank (sv) == 0)
 		num_elements = 1;
 	      else
@@ -295,6 +355,12 @@ ffeequiv_layout_local_ (ffeequiv eq)
 		  continue;
 		}
 
+	      /* The last symbol we see with a zero offset must therefore
+		 (by static deduction) be the first-listed "rooted" item
+		 in the EQUIVALENCE statements pertaining to this area.  */
+	      if (var_offset == 0)
+		ffestorag_set_symbol (st, sv);
+
 	      if ((vst = ffesymbol_storage (sv)) == NULL)
 		{		/* Create new ffestorag object, extend equiv
 				   area. */
@@ -341,11 +407,13 @@ ffeequiv_layout_local_ (ffeequiv eq)
 		    }
 		}
 	    }			/* (For every equivalence item in the list) */
+#if FFEEQUIV_DEBUG
+	  fprintf (stderr, ")\n");
+#endif
 	  ffebld_set_head (list, NULL);	/* Don't do this list again. */
 	}			/* (For every equivalence list in the list of
 				   equivs) */
-    }
-  while (new_storage && need_storage);
+    } while (new_storage && need_storage);
 
   ffeequiv_kill (eq);		/* Fully processed, no longer needed. */
 
@@ -370,8 +438,8 @@ ffeequiv_layout_local_ (ffeequiv eq)
    have the same first storage unit and so return the same offset.  */
 
 static bool
-ffeequiv_offset_ (ffetargetOffset *offset, ffesymbol s, ffebld expr,
-		  bool subtract, ffetargetOffset adjust)
+ffeequiv_offset_ (ffetargetOffset *offset, ffesymbol s UNUSED,
+		  ffebld expr, bool subtract, ffetargetOffset adjust)
 {
   ffetargetIntegerDefault value = 0;
   ffetargetOffset cval;		/* Converted value. */
@@ -828,9 +896,9 @@ ffeequiv_layout_cblock (ffestorag st)
 		}
 
 	      /* We now know the root symbol and the operating offset of that
-	         root into the common area.  The other expressions in the
-	         list all identify an initial storage unit that must have the
-	         same offset. */
+		 root into the common area.  The other expressions in the
+		 list all identify an initial storage unit that must have the
+		 same offset. */
 
 	      for (var = ffebld_head (item);
 		   var != NULL;
