@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
 */
 
@@ -58,18 +59,21 @@ struct _ffeintrin_imp_
     char *name;			/* Name of implementation. */
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
     ffecomGfrt gfrt;		/* gfrt index in library. */
-#endif
+#endif	/* FFECOM_targetCURRENT == FFECOM_targetGCC */
     ffeinfoBasictype basictype;
     ffeinfoKindtype kindtype;
     ffetargetCharacterSize size;
     ffebad (*check) (ffebld arglist);
   };
 
+static ffebad ffeintrin_check_0_ (ffebld arglist);
 static ffebad ffeintrin_check_1_ (ffebld arglist, ffebld *xarg1);
 static ffebad ffeintrin_check_1or2_ (ffebld arglist, ffebld *xarg1,
 				     ffebld *xarg2);
 static ffebad ffeintrin_check_2_ (ffebld arglist, ffebld *xarg1,
 				  ffebld *xarg2);
+static ffebad ffeintrin_check_2or3_ (ffebld arglist, ffebld *xarg1,
+				  ffebld *xarg2, ffebld *xarg3);
 static ffebad ffeintrin_check_3_ (ffebld arglist, ffebld *xarg1,
 				  ffebld *xarg2, ffebld *xarg3);
 static ffebad ffeintrin_check_5_ (ffebld arglist, ffebld *xarg1,
@@ -82,7 +86,10 @@ static ffebad ffeintrin_check_cmplx_1_ (ffebld arglist);
 static ffebad ffeintrin_check_cmplx_1or2_ (ffebld arglist);
 static ffebad ffeintrin_check_dcmplx_1_ (ffebld arglist);
 static ffebad ffeintrin_check_dcmplx_1or2_ (ffebld arglist);
+static ffebad ffeintrin_check_getarg_ (ffebld arglist);
+static ffebad ffeintrin_check_getenv_ (ffebld arglist);
 static ffebad ffeintrin_check_int_1_ (ffebld arglist);
+static ffebad ffeintrin_check_int_1_o_ (ffebld arglist);
 static ffebad ffeintrin_check_int_1or2_ (ffebld arglist);
 static ffebad ffeintrin_check_int_2_ (ffebld arglist);
 static ffebad ffeintrin_check_int_2p_ (ffebld arglist);
@@ -106,18 +113,24 @@ static ffebad ffeintrin_check_realdbl_1_ (ffebld arglist);
 static ffebad ffeintrin_check_realdbl_1or2_ (ffebld arglist);
 static ffebad ffeintrin_check_realdbl_2_ (ffebld arglist);
 static ffebad ffeintrin_check_realdbl_2p_ (ffebld arglist);
+static ffebad ffeintrin_check_signal_ (ffebld arglist);
+static ffebad ffeintrin_check_system_ (ffebld arglist);
 static ffebad ffeintrin_check_void_ (ffebld arglist);
 static int ffeintrin_cmp_name_ (const void *name, const void *intrinsic);
 
+#define ffeintrin_check_exit_ ffeintrin_check_int_1_o_
+#define ffeintrin_check_flush_ ffeintrin_check_int_1_o_
+
 static struct _ffeintrin_name_ ffeintrin_names_[]
 =
 {				/* Alpha order. */
+  {"ABORT", "abort", "Abort", FFEINTRIN_genNONE, FFEINTRIN_specABORT,},	/* UNIX */
   {"ABS", "abs", "Abs", FFEINTRIN_genABS, FFEINTRIN_specABS,},
   {"ACHAR", "achar", "AChar", FFEINTRIN_genACHAR, FFEINTRIN_specNONE,},	/* F90, F2C */
   {"ACOS", "acos", "ACos", FFEINTRIN_genACOS, FFEINTRIN_specACOS,},
   {"ACOSD", "acosd", "ACosD", FFEINTRIN_genACOSD, FFEINTRIN_specACOSD,},	/* VXT */
-{"ADJUSTL", "adjustl", "AdjustL", FFEINTRIN_genADJUSTL, FFEINTRIN_specNONE,},	/* F90 */
-{"ADJUSTR", "adjustr", "AdjustR", FFEINTRIN_genADJUSTR, FFEINTRIN_specNONE,},	/* F90 */
+  {"ADJUSTL", "adjustl", "AdjustL", FFEINTRIN_genADJUSTL, FFEINTRIN_specNONE,},	/* F90 */
+  {"ADJUSTR", "adjustr", "AdjustR", FFEINTRIN_genADJUSTR, FFEINTRIN_specNONE,},	/* F90 */
   {"AIMAG", "aimag", "AImag", FFEINTRIN_genAIMAG, FFEINTRIN_specAIMAG,},
   {"AIMAX0", "aimax0", "AIMax0", FFEINTRIN_genNONE, FFEINTRIN_specAIMAX0,},	/* VXT */
   {"AIMIN0", "aimin0", "AIMin0", FFEINTRIN_genNONE, FFEINTRIN_specAIMIN0,},	/* VXT */
@@ -155,7 +168,7 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"CDLOG", "cdlog", "CDLog", FFEINTRIN_genNONE, FFEINTRIN_specCDLOG,},	/* VXT */
   {"CDSIN", "cdsin", "CDSin", FFEINTRIN_genNONE, FFEINTRIN_specCDSIN,},	/* VXT */
   {"CDSQRT", "cdsqrt", "CDSqRt", FFEINTRIN_genNONE, FFEINTRIN_specCDSQRT,},	/* VXT */
-{"CEILING", "ceiling", "Ceiling", FFEINTRIN_genCEILING, FFEINTRIN_specNONE,},	/* F90 */
+  {"CEILING", "ceiling", "Ceiling", FFEINTRIN_genCEILING, FFEINTRIN_specNONE,},	/* F90 */
   {"CEXP", "cexp", "CExp", FFEINTRIN_genNONE, FFEINTRIN_specCEXP,},
   {"CHAR", "char", "Char", FFEINTRIN_genCHAR, FFEINTRIN_specCHAR,},
   {"CLOG", "clog", "CLog", FFEINTRIN_genNONE, FFEINTRIN_specCLOG,},
@@ -175,7 +188,7 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"DASIND", "dasind", "DASinD", FFEINTRIN_genNONE, FFEINTRIN_specDASIND,},	/* VXT */
   {"DATAN", "datan", "DATan", FFEINTRIN_genNONE, FFEINTRIN_specDATAN,},
   {"DATAN2", "datan2", "DATan2", FFEINTRIN_genNONE, FFEINTRIN_specDATAN2,},
-{"DATAN2D", "datan2d", "DATan2D", FFEINTRIN_genNONE, FFEINTRIN_specDATAN2D,},	/* VXT */
+  {"DATAN2D", "datan2d", "DATan2D", FFEINTRIN_genNONE, FFEINTRIN_specDATAN2D,},	/* VXT */
   {"DATAND", "datand", "DATanD", FFEINTRIN_genNONE, FFEINTRIN_specDATAND,},	/* VXT */
   {"DATE_AND_TIME", "date_and_time", "Date_and_Time", FFEINTRIN_genNONE, FFEINTRIN_specDATE_AND_TIME,},	/* F90 */
   {"DBLE", "dble", "Dble", FFEINTRIN_genDBLE, FFEINTRIN_specDBLE,},
@@ -186,6 +199,8 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"DCOSD", "dcosd", "DCosD", FFEINTRIN_genNONE, FFEINTRIN_specDCOSD,},	/* VXT */
   {"DCOSH", "dcosh", "DCosH", FFEINTRIN_genNONE, FFEINTRIN_specDCOSH,},
   {"DDIM", "ddim", "DDim", FFEINTRIN_genNONE, FFEINTRIN_specDDIM,},
+  {"DERF", "derf", "DErF", FFEINTRIN_genNONE, FFEINTRIN_specDERF,},	/* UNIX */
+  {"DERFC", "derfc", "DErFC", FFEINTRIN_genNONE, FFEINTRIN_specDERFC,},	/* UNIX */
   {"DEXP", "dexp", "DExp", FFEINTRIN_genNONE, FFEINTRIN_specDEXP,},
   {"DFLOAT", "dfloat", "DFloat", FFEINTRIN_genDFLOAT, FFEINTRIN_specNONE,},	/* F2C, VXT */
   {"DFLOTI", "dfloti", "DFlotI", FFEINTRIN_genNONE, FFEINTRIN_specDFLOTI,},	/* VXT */
@@ -211,14 +226,18 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"DTAN", "dtan", "DTan", FFEINTRIN_genNONE, FFEINTRIN_specDTAN,},
   {"DTAND", "dtand", "DTanD", FFEINTRIN_genNONE, FFEINTRIN_specDTAND,},	/* VXT */
   {"DTANH", "dtanh", "DTanH", FFEINTRIN_genNONE, FFEINTRIN_specDTANH,},
-{"EOSHIFT", "eoshift", "EOShift", FFEINTRIN_genEOSHIFT, FFEINTRIN_specNONE,},	/* F90 */
-{"EPSILON", "epsilon", "Epsilon", FFEINTRIN_genEPSILON, FFEINTRIN_specNONE,},	/* F90 */
+  {"EOSHIFT", "eoshift", "EOShift", FFEINTRIN_genEOSHIFT, FFEINTRIN_specNONE,},	/* F90 */
+  {"EPSILON", "epsilon", "Epsilon", FFEINTRIN_genEPSILON, FFEINTRIN_specNONE,},	/* F90 */
+  {"ERF", "erf", "ErF", FFEINTRIN_genNONE, FFEINTRIN_specERF,},	/* UNIX */
+  {"ERFC", "erfc", "ErFC", FFEINTRIN_genNONE, FFEINTRIN_specERFC,},	/* UNIX */
+  {"EXIT", "exit", "Exit", FFEINTRIN_genNONE, FFEINTRIN_specEXIT,},	/* UNIX */
   {"EXP", "exp", "Exp", FFEINTRIN_genEXP, FFEINTRIN_specEXP,},
   {"EXPONENT", "exponent", "Exponent", FFEINTRIN_genEXPONENT, FFEINTRIN_specNONE,},	/* F90 */
   {"FLOAT", "float", "Float", FFEINTRIN_genFLOAT, FFEINTRIN_specFLOAT,},
   {"FLOATI", "floati", "FloatI", FFEINTRIN_genNONE, FFEINTRIN_specFLOATI,},	/* VXT */
   {"FLOATJ", "floatj", "FloatJ", FFEINTRIN_genNONE, FFEINTRIN_specFLOATJ,},	/* VXT */
   {"FLOOR", "floor", "Floor", FFEINTRIN_genFLOOR, FFEINTRIN_specNONE,},	/* F90 */
+  {"FLUSH", "flush", "Flush", FFEINTRIN_genNONE, FFEINTRIN_specFLUSH,},	/* UNIX */
   {"FPABSP", "fpabsp", "FPAbsP", FFEINTRIN_genFPABSP, FFEINTRIN_specNONE,},	/* F2C */
   {"FPEXPN", "fpexpn", "FPExpn", FFEINTRIN_genFPEXPN, FFEINTRIN_specNONE,},	/* F2C */
   {"FPFRAC", "fpfrac", "FPFrac", FFEINTRIN_genFPFRAC, FFEINTRIN_specNONE,},	/* F2C */
@@ -226,10 +245,13 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"FPRRSP", "fprrsp", "FPRRSp", FFEINTRIN_genFPRRSP, FFEINTRIN_specNONE,},	/* F2C */
   {"FPSCAL", "fpscal", "FPScal", FFEINTRIN_genFPSCAL, FFEINTRIN_specNONE,},	/* F2C */
   {"FRACTION", "fraction", "Fraction", FFEINTRIN_genFRACTION, FFEINTRIN_specNONE,},	/* F90 */
+  {"GETARG", "getarg", "GetArg", FFEINTRIN_genNONE, FFEINTRIN_specGETARG,},	/* UNIX */
+  {"GETENV", "getenv", "GetEnv", FFEINTRIN_genNONE, FFEINTRIN_specGETENV,},	/* UNIX */
   {"HUGE", "huge", "Huge", FFEINTRIN_genHUGE, FFEINTRIN_specNONE,},	/* F90 */
   {"IABS", "iabs", "IAbs", FFEINTRIN_genIABS, FFEINTRIN_specIABS,},
   {"IACHAR", "iachar", "IAChar", FFEINTRIN_genIACHAR, FFEINTRIN_specNONE,},	/* F90, F2C */
   {"IAND", "iand", "IAnd", FFEINTRIN_genIAND, FFEINTRIN_specNONE,},	/* F90, VXT */
+  {"IARGC", "iargc", "IArgC", FFEINTRIN_genNONE, FFEINTRIN_specIARGC,},	/* UNIX */
   {"IBCLR", "ibclr", "IBClr", FFEINTRIN_genIBCLR, FFEINTRIN_specNONE,},	/* F90, VXT */
   {"IBITS", "ibits", "IBits", FFEINTRIN_genIBITS, FFEINTRIN_specNONE,},	/* F90, VXT */
   {"IBSET", "ibset", "IBSet", FFEINTRIN_genIBSET, FFEINTRIN_specNONE,},	/* F90, VXT */
@@ -254,7 +276,7 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"IIQINT", "iiqint", "IIQint", FFEINTRIN_genNONE, FFEINTRIN_specIIQINT,},	/* VXT */
   {"IIQNNT", "iiqnnt", "IIQNnt", FFEINTRIN_genNONE, FFEINTRIN_specIIQNNT,},	/* VXT */
   {"IISHFT", "iishft", "IIShft", FFEINTRIN_genNONE, FFEINTRIN_specNONE,},	/* VXT */
-{"IISHFTC", "iishftc", "IIShftC", FFEINTRIN_genNONE, FFEINTRIN_specIISHFTC,},	/* VXT */
+  {"IISHFTC", "iishftc", "IIShftC", FFEINTRIN_genNONE, FFEINTRIN_specIISHFTC,},	/* VXT */
   {"IISIGN", "iisign", "IISign", FFEINTRIN_genNONE, FFEINTRIN_specIISIGN,},	/* VXT */
   {"IMAG", "imag", "Imag", FFEINTRIN_genIMAG, FFEINTRIN_spec_IMAG_C_F2C,},	/* F2C */
   {"IMAX0", "imax0", "IMax0", FFEINTRIN_genNONE, FFEINTRIN_specIMAX0,},	/* VXT */
@@ -286,7 +308,7 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"JIQINT", "jiqint", "JIQint", FFEINTRIN_genNONE, FFEINTRIN_specJIQINT,},	/* VXT */
   {"JIQNNT", "jiqnnt", "JIQNnt", FFEINTRIN_genNONE, FFEINTRIN_specJIQNNT,},	/* VXT */
   {"JISHFT", "jishft", "JIShft", FFEINTRIN_genNONE, FFEINTRIN_specJISHFT,},	/* VXT */
-{"JISHFTC", "jishftc", "JIShftC", FFEINTRIN_genNONE, FFEINTRIN_specJISHFTC,},	/* VXT */
+  {"JISHFTC", "jishftc", "JIShftC", FFEINTRIN_genNONE, FFEINTRIN_specJISHFTC,},	/* VXT */
   {"JISIGN", "jisign", "JISign", FFEINTRIN_genNONE, FFEINTRIN_specJISIGN,},	/* VXT */
   {"JMAX0", "jmax0", "JMax0", FFEINTRIN_genNONE, FFEINTRIN_specJMAX0,},	/* VXT */
   {"JMAX1", "jmax1", "JMax1", FFEINTRIN_genNONE, FFEINTRIN_specJMAX1,},	/* VXT */
@@ -307,7 +329,7 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"LOC", "loc", "Loc", FFEINTRIN_genNONE, FFEINTRIN_specLOC,},	/* VXT */
   {"LOG", "log", "Log", FFEINTRIN_genLOG, FFEINTRIN_specNONE,},
   {"LOG10", "log10", "Log10", FFEINTRIN_genLOG10, FFEINTRIN_specNONE,},
-{"LOGICAL", "logical", "Logical", FFEINTRIN_genLOGICAL, FFEINTRIN_specNONE,},	/* F90 */
+  {"LOGICAL", "logical", "Logical", FFEINTRIN_genLOGICAL, FFEINTRIN_specNONE,},	/* F90 */
   {"LSHIFT", "lshift", "LShift", FFEINTRIN_genLSHIFT, FFEINTRIN_specNONE,},	/* F2C */
   {"MATMUL", "matmul", "MatMul", FFEINTRIN_genMATMUL, FFEINTRIN_specNONE,},	/* F90 */
   {"MAX", "max", "Max", FFEINTRIN_genMAX, FFEINTRIN_specNONE,},
@@ -326,14 +348,14 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"MOD", "mod", "Mod", FFEINTRIN_genMOD, FFEINTRIN_specMOD,},
   {"MODULO", "modulo", "Modulo", FFEINTRIN_genMODULO, FFEINTRIN_specNONE,},	/* F90 */
   {"MVBITS", "mvbits", "MvBits", FFEINTRIN_genMVBITS, FFEINTRIN_specNONE,},	/* F90 */
-{"NEAREST", "nearest", "Nearest", FFEINTRIN_genNEAREST, FFEINTRIN_specNONE,},	/* F90 */
+  {"NEAREST", "nearest", "Nearest", FFEINTRIN_genNEAREST, FFEINTRIN_specNONE,},	/* F90 */
   {"NINT", "nint", "NInt", FFEINTRIN_genNINT, FFEINTRIN_specNINT,},
   {"NOT", "not", "Not", FFEINTRIN_genNOT, FFEINTRIN_specNONE,},	/* F2C, F90, VXT */
   {"OR", "or", "Or", FFEINTRIN_genOR, FFEINTRIN_specNONE,},	/* F2C */
   {"PACK", "pack", "Pack", FFEINTRIN_genPACK, FFEINTRIN_specNONE,},	/* F90 */
   {"PRECISION", "precision", "Precision", FFEINTRIN_genPRECISION, FFEINTRIN_specNONE,},	/* F90 */
-{"PRESENT", "present", "Present", FFEINTRIN_genPRESENT, FFEINTRIN_specNONE,},	/* F90 */
-{"PRODUCT", "product", "Product", FFEINTRIN_genPRODUCT, FFEINTRIN_specNONE,},	/* F90 */
+  {"PRESENT", "present", "Present", FFEINTRIN_genPRESENT, FFEINTRIN_specNONE,},	/* F90 */
+  {"PRODUCT", "product", "Product", FFEINTRIN_genPRODUCT, FFEINTRIN_specNONE,},	/* F90 */
   {"QABS", "qabs", "QAbs", FFEINTRIN_genNONE, FFEINTRIN_specQABS,},	/* VXT */
   {"QACOS", "qacos", "QACos", FFEINTRIN_genNONE, FFEINTRIN_specQACOS,},	/* VXT */
   {"QACOSD", "qacosd", "QACosD", FFEINTRIN_genNONE, FFEINTRIN_specQACOSD,},	/* VXT */
@@ -341,7 +363,7 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"QASIND", "qasind", "QASinD", FFEINTRIN_genNONE, FFEINTRIN_specQASIND,},	/* VXT */
   {"QATAN", "qatan", "QATan", FFEINTRIN_genNONE, FFEINTRIN_specQATAN,},	/* VXT */
   {"QATAN2", "qatan2", "QATan2", FFEINTRIN_genNONE, FFEINTRIN_specQATAN2,},	/* VXT */
-{"QATAN2D", "qatan2d", "QATan2D", FFEINTRIN_genNONE, FFEINTRIN_specQATAN2D,},	/* VXT */
+  {"QATAN2D", "qatan2d", "QATan2D", FFEINTRIN_genNONE, FFEINTRIN_specQATAN2D,},	/* VXT */
   {"QATAND", "qatand", "QATanD", FFEINTRIN_genNONE, FFEINTRIN_specQATAND,},	/* VXT */
   {"QCOS", "qcos", "QCos", FFEINTRIN_genNONE, FFEINTRIN_specQCOS,},	/* VXT */
   {"QCOSD", "qcosd", "QCosD", FFEINTRIN_genNONE, FFEINTRIN_specQCOSD,},	/* VXT */
@@ -371,7 +393,7 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"RANGE", "range", "Range", FFEINTRIN_genRANGE, FFEINTRIN_specNONE,},	/* F90 */
   {"REAL", "real", "Real", FFEINTRIN_genREAL, FFEINTRIN_specREAL,},
   {"REPEAT", "repeat", "Repeat", FFEINTRIN_genREPEAT, FFEINTRIN_specNONE,},	/* F90 */
-{"RESHAPE", "reshape", "Reshape", FFEINTRIN_genRESHAPE, FFEINTRIN_specNONE,},	/* F90 */
+  {"RESHAPE", "reshape", "Reshape", FFEINTRIN_genRESHAPE, FFEINTRIN_specNONE,},	/* F90 */
   {"RRSPACING", "rrspacing", "RRSpacing", FFEINTRIN_genRRSPACING, FFEINTRIN_specNONE,},	/* F90 */
   {"RSHIFT", "rshift", "RShift", FFEINTRIN_genRSHIFT, FFEINTRIN_specNONE,},	/* F2C */
   {"SCALE", "scale", "Scale", FFEINTRIN_genSCALE, FFEINTRIN_specNONE,},	/* F90 */
@@ -381,15 +403,17 @@ static struct _ffeintrin_name_ ffeintrin_names_[]
   {"SET_EXPONENT", "set_exponent", "Set_Exponent", FFEINTRIN_genSET_EXPONENT, FFEINTRIN_specNONE,},	/* F90 */
   {"SHAPE", "shape", "Shape", FFEINTRIN_genSHAPE, FFEINTRIN_specNONE,},	/* F90 */
   {"SIGN", "sign", "Sign", FFEINTRIN_genSIGN, FFEINTRIN_specSIGN,},
+  {"SIGNAL", "signal", "Signal", FFEINTRIN_genNONE, FFEINTRIN_specSIGNAL,},	/* UNIX */
   {"SIN", "sin", "Sin", FFEINTRIN_genSIN, FFEINTRIN_specSIN,},
   {"SIND", "sind", "SinD", FFEINTRIN_genSIND, FFEINTRIN_specSIND,},	/* VXT */
   {"SINH", "sinh", "SinH", FFEINTRIN_genSINH, FFEINTRIN_specSINH,},
   {"SNGL", "sngl", "Sngl", FFEINTRIN_genNONE, FFEINTRIN_specSNGL,},
   {"SNGLQ", "snglq", "SnglQ", FFEINTRIN_genNONE, FFEINTRIN_specSNGLQ,},	/* VXT */
-{"SPACING", "spacing", "Spacing", FFEINTRIN_genSPACING, FFEINTRIN_specNONE,},	/* F90 */
+  {"SPACING", "spacing", "Spacing", FFEINTRIN_genSPACING, FFEINTRIN_specNONE,},	/* F90 */
   {"SPREAD", "spread", "Spread", FFEINTRIN_genSPREAD, FFEINTRIN_specNONE,},	/* F90 */
   {"SQRT", "sqrt", "SqRt", FFEINTRIN_genSQRT, FFEINTRIN_specSQRT,},
   {"SUM", "sum", "Sum", FFEINTRIN_genSUM, FFEINTRIN_specNONE,},	/* F90 */
+  {"SYSTEM", "system", "System", FFEINTRIN_genNONE, FFEINTRIN_specSYSTEM,},	/* UNIX */
   {"SYSTEM_CLOCK", "system_clock", "System_Clock", FFEINTRIN_genNONE, FFEINTRIN_specSYSTEM_CLOCK,},	/* F90 */
   {"TAN", "tan", "Tan", FFEINTRIN_genTAN, FFEINTRIN_specTAN,},
   {"TAND", "tand", "TanD", FFEINTRIN_genTAND, FFEINTRIN_specTAND,},	/* VXT */
@@ -434,10 +458,11 @@ static struct _ffeintrin_imp_ ffeintrin_imps_[]
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
 #define DEFIMP(CODE,NAME,GFRT,RETURNS,EXPECTS) \
       { NAME, GFRT, RETURNS, EXPECTS },
-#endif
-#if FFECOM_targetCURRENT == FFECOM_targetFFE
+#elif FFECOM_targetCURRENT == FFECOM_targetFFE
 #define DEFIMP(CODE,NAME,GFRT,RETURNS,EXPECTS) \
       { NAME, RETURNS, EXPECTS },
+#else
+#error
 #endif
 #define DEFSPEC(CODE,NAME,CALLABLE,FAMILY,IMP)
 #include "intrin.def"
@@ -461,6 +486,15 @@ static struct _ffeintrin_spec_ ffeintrin_specs_[]
 };
 
 
+static ffebad
+ffeintrin_check_0_ (ffebld arglist)
+{
+  if (arglist != NULL)
+    return FFEBAD_INTRINSIC_TOOMANY;
+
+  return FFEBAD;
+}
+
 static ffebad
 ffeintrin_check_1_ (ffebld arglist, ffebld *xarg1)
 {
@@ -531,6 +565,39 @@ ffeintrin_check_2_ (ffebld arglist, ffebld *xarg1, ffebld *xarg2)
   return FFEBAD;
 }
 
+static ffebad
+ffeintrin_check_2or3_ (ffebld arglist, ffebld *xarg1, ffebld *xarg2,
+		    ffebld *xarg3)
+{
+  ffebld arg1;
+  ffebld arg2;
+  ffebld arg3;
+
+  arg1 = arglist;
+  if (arg1 == NULL)
+    return FFEBAD_INTRINSIC_TOOFEW;
+
+  arg2 = ffebld_trail (arg1);
+  if (arg2 == NULL)
+    return FFEBAD_INTRINSIC_TOOFEW;
+
+  arg3 = ffebld_trail (arg2);
+  if ((arg3 != NULL)
+      && (ffebld_trail (arg3) != NULL))
+    return FFEBAD_INTRINSIC_TOOMANY;
+
+  if (((arg1 = ffebld_head (arg1)) == NULL)
+      || ((arg2 = ffebld_head (arg2)) == NULL)
+      || ((arg3 != NULL)
+	  && ((arg3 = ffebld_head (arg3)) == NULL)))
+    return FFEBAD_INTRINSIC_REF;
+
+  *xarg1 = arg1;
+  *xarg2 = arg2;
+  *xarg3 = arg3;
+  return FFEBAD;
+}
+
 static ffebad
 ffeintrin_check_3_ (ffebld arglist, ffebld *xarg1, ffebld *xarg2,
 		    ffebld *xarg3)
@@ -785,6 +852,95 @@ ffeintrin_check_dcmplx_1or2_ (ffebld arglist)
   return FFEBAD;		/* Ok. */
 }
 
+static ffebad
+ffeintrin_check_getarg_ (ffebld arglist)
+{
+  ffebld arg1;
+  ffebld arg2;
+  ffeinfo info;
+  ffebad bad;
+
+  bad = ffeintrin_check_2_ (arglist, &arg1, &arg2);
+  if (bad != FFEBAD)
+    return bad;
+
+  info = ffebld_info (arg1);
+  if ((ffeinfo_basictype (info) != FFEINFO_basictypeINTEGER)
+      || (ffeinfo_kindtype (info) != FFEINFO_kindtypeINTEGERDEFAULT)
+      || (ffeinfo_rank (info) != 0)
+      || (ffeinfo_kind (info) != FFEINFO_kindENTITY))
+    return FFEBAD_INTRINSIC_REF;
+
+  info = ffebld_info (arg2);
+  if ((ffeinfo_basictype (info) != FFEINFO_basictypeCHARACTER)
+      || (ffeinfo_kindtype (info) != FFEINFO_kindtypeCHARACTERDEFAULT)
+      || (ffeinfo_rank (info) != 0)
+      || (ffeinfo_kind (info) != FFEINFO_kindENTITY)
+      || ((ffebld_op (arg2) != FFEBLD_opSYMTER)
+	  && (ffebld_op (arg2) != FFEBLD_opARRAYREF)
+	  && (ffebld_op (arg2) != FFEBLD_opSUBSTR)))
+    return FFEBAD_INTRINSIC_REF;
+
+  return FFEBAD;		/* Ok. */
+}
+
+static ffebad
+ffeintrin_check_getenv_ (ffebld arglist)
+{
+  ffebld arg1;
+  ffebld arg2;
+  ffeinfo info;
+  ffebad bad;
+
+  bad = ffeintrin_check_2_ (arglist, &arg1, &arg2);
+  if (bad != FFEBAD)
+    return bad;
+
+  info = ffebld_info (arg1);
+  if ((ffeinfo_basictype (info) != FFEINFO_basictypeCHARACTER)
+      || (ffeinfo_kindtype (info) != FFEINFO_kindtypeCHARACTERDEFAULT)
+      || (ffeinfo_rank (info) != 0)
+      || (ffeinfo_kind (info) != FFEINFO_kindENTITY))
+    return FFEBAD_INTRINSIC_REF;
+
+  info = ffebld_info (arg2);
+  if ((ffeinfo_basictype (info) != FFEINFO_basictypeCHARACTER)
+      || (ffeinfo_kindtype (info) != FFEINFO_kindtypeCHARACTERDEFAULT)
+      || (ffeinfo_rank (info) != 0)
+      || (ffeinfo_kind (info) != FFEINFO_kindENTITY)
+      || ((ffebld_op (arg2) != FFEBLD_opSYMTER)
+	  && (ffebld_op (arg2) != FFEBLD_opARRAYREF)
+	  && (ffebld_op (arg2) != FFEBLD_opSUBSTR)))
+    return FFEBAD_INTRINSIC_REF;
+
+  return FFEBAD;		/* Ok. */
+}
+
+static ffebad
+ffeintrin_check_int_1_o_ (ffebld arglist)
+{
+  ffebld arg1;
+  ffeinfo info;
+  ffebad bad;
+
+  bad = ffeintrin_check_0_ (arglist);
+  if (bad == FFEBAD)
+    return bad;
+
+  bad = ffeintrin_check_1_ (arglist, &arg1);
+  if (bad != FFEBAD)
+    return bad;
+
+  info = ffebld_info (arg1);
+  if ((ffeinfo_basictype (info) != FFEINFO_basictypeINTEGER)
+      || (ffeinfo_kindtype (info) != FFEINFO_kindtypeINTEGERDEFAULT)
+      || (ffeinfo_rank (info) != 0)
+      || (ffeinfo_kind (info) != FFEINFO_kindENTITY))
+    return FFEBAD_INTRINSIC_REF;
+
+  return FFEBAD;		/* Ok. */
+}
+
 static ffebad
 ffeintrin_check_int_1_ (ffebld arglist)
 {
@@ -1350,7 +1506,100 @@ ffeintrin_check_realdbl_2p_ (ffebld arglist)
 }
 
 static ffebad
-ffeintrin_check_void_ (ffebld arglist)
+ffeintrin_check_signal_ (ffebld arglist)
+{
+  ffebld arg1;
+  ffebld arg2;
+  ffebld arg3;
+  ffeinfo info;
+  ffebad bad;
+
+  bad = ffeintrin_check_2or3_ (arglist, &arg1, &arg2, &arg3);
+  if (bad != FFEBAD)
+    return bad;
+
+  info = ffebld_info (arg1);
+  if ((ffeinfo_basictype (info) != FFEINFO_basictypeINTEGER)
+      || (ffeinfo_kindtype (info) != FFEINFO_kindtypeINTEGERDEFAULT)
+      || (ffeinfo_rank (info) != 0)
+      || (ffeinfo_kind (info) != FFEINFO_kindENTITY))
+    return FFEBAD_INTRINSIC_REF;
+
+  /* Second arg is either a dummy/external subroutine or an integer
+     (for SIG_DFL, SIG_IGN, SIG_ERR).  */
+
+  info = ffebld_info (arg2);
+  if (((((ffeinfo_basictype (info) != FFEINFO_basictypeNONE)
+	 || (ffeinfo_kindtype (info) != FFEINFO_kindtypeNONE)
+	 || (ffeinfo_kind (info) != FFEINFO_kindSUBROUTINE))
+	&& ((ffeinfo_basictype (info) != FFEINFO_basictypeINTEGER)
+	    || (ffeinfo_kindtype (info) != FFEINFO_kindtypeINTEGERDEFAULT)
+	    || (ffeinfo_kind (info) != FFEINFO_kindFUNCTION))
+	&& (ffeinfo_kind (info) != FFEINFO_kindNONE))	/* Procedure. */
+       || (ffeinfo_rank (info) != 0)
+       || ((ffeinfo_where (info) != FFEINFO_whereDUMMY)
+	   && (ffeinfo_where (info) != FFEINFO_whereGLOBAL)))
+      && ((ffeinfo_basictype (info) != FFEINFO_basictypeINTEGER)
+	  || (ffeinfo_kindtype (info) != FFEINFO_kindtypeINTEGERDEFAULT)
+	  || (ffeinfo_rank (info) != 0)
+	  || (ffeinfo_kind (info) != FFEINFO_kindENTITY)))
+    return FFEBAD_INTRINSIC_REF;
+
+  /* Third arg, if present, is a place to put the returned value.  */
+
+  if (arg3 != NULL)
+    {
+      info = ffebld_info (arg3);
+      if ((ffeinfo_basictype (info) != FFEINFO_basictypeINTEGER)
+	  || (ffeinfo_kindtype (info) != FFEINFO_kindtypeINTEGERDEFAULT)
+	  || (ffeinfo_rank (info) != 0)
+	  || (ffeinfo_kind (info) != FFEINFO_kindENTITY)
+	  || ((ffebld_op (arg3) != FFEBLD_opSYMTER)
+	      && (ffebld_op (arg3) != FFEBLD_opARRAYREF)))
+	return FFEBAD_INTRINSIC_REF;
+    }
+
+  return FFEBAD;		/* Ok. */
+}
+
+static ffebad
+ffeintrin_check_system_ (ffebld arglist)
+{
+  ffebld arg1;
+  ffebld arg2;
+  ffeinfo info;
+  ffebad bad;
+
+  bad = ffeintrin_check_1or2_ (arglist, &arg1, &arg2);
+  if (bad != FFEBAD)
+    return bad;
+
+  info = ffebld_info (arg1);
+  if ((ffeinfo_basictype (info) != FFEINFO_basictypeCHARACTER)
+      || (ffeinfo_kindtype (info) != FFEINFO_kindtypeCHARACTERDEFAULT)
+      || (ffeinfo_rank (info) != 0)
+      || (ffeinfo_kind (info) != FFEINFO_kindENTITY))
+    return FFEBAD_INTRINSIC_REF;
+
+  /* Second arg, if present, is a place to put the returned value.  */
+
+  if (arg2 != NULL)
+    {
+      info = ffebld_info (arg2);
+      if ((ffeinfo_basictype (info) != FFEINFO_basictypeINTEGER)
+	  || (ffeinfo_kindtype (info) != FFEINFO_kindtypeINTEGERDEFAULT)
+	  || (ffeinfo_rank (info) != 0)
+	  || (ffeinfo_kind (info) != FFEINFO_kindENTITY)
+	  || ((ffebld_op (arg2) != FFEBLD_opSYMTER)
+	      && (ffebld_op (arg2) != FFEBLD_opARRAYREF)))
+	return FFEBAD_INTRINSIC_REF;
+    }
+
+  return FFEBAD;		/* Ok. */
+}
+
+static ffebad
+ffeintrin_check_void_ (ffebld arglist UNUSED)
 {
   return FFEBAD;		/* Ok. */
 }
@@ -1422,7 +1671,7 @@ ffeintrin_fulfill_generic (ffebld *expr, ffeinfo *info, ffelexToken t)
   error = FFEBAD;
 
   for (i = 0;
-       (i < ARRAY_SIZE (ffeintrin_gens_[gen].specs))
+       (((size_t) i) < ARRAY_SIZE (ffeintrin_gens_[gen].specs))
        && ((tspec = ffeintrin_gens_[gen].specs[i]) != FFEINTRIN_specNONE);
        ++i)
     {
@@ -1646,7 +1895,7 @@ ffeintrin_init_0 ()
   assert (FFEINTRIN_imp == ARRAY_SIZE (ffeintrin_imps_));
   assert (FFEINTRIN_spec == ARRAY_SIZE (ffeintrin_specs_));
 
-  for (i = 1; i < ARRAY_SIZE (ffeintrin_names_); ++i)
+  for (i = 1; ((size_t) i) < ARRAY_SIZE (ffeintrin_names_); ++i)
     {				/* Make sure binary-searched list is in alpha
 				   order. */
       if (strcmp (ffeintrin_names_[i - 1].name_uc,
@@ -1654,7 +1903,7 @@ ffeintrin_init_0 ()
 	assert ("name list out of order" == NULL);
     }
 
-  for (i = 0; i < ARRAY_SIZE (ffeintrin_names_); ++i)
+  for (i = 0; ((size_t) i) < ARRAY_SIZE (ffeintrin_names_); ++i)
     {
       p1 = ffeintrin_names_[i].name_uc;
       p2 = ffeintrin_names_[i].name_lc;
@@ -1747,7 +1996,7 @@ ffeintrin_is_intrinsic (char *name, ffelexToken t, bool explicit,
       bool ok = FALSE;
 
       for (i = 0;
-	   (i < ARRAY_SIZE (ffeintrin_gens_[gen].specs))
+	   (((size_t) i) < ARRAY_SIZE (ffeintrin_gens_[gen].specs))
 	   && ((tspec
 		= ffeintrin_gens_[gen].specs[i]) != FFEINTRIN_specNONE);
 	   ++i)
@@ -1808,7 +2057,7 @@ ffeintrin_is_intrinsic (char *name, ffelexToken t, bool explicit,
   if ((gen == FFEINTRIN_genNONE) && (spec == FFEINTRIN_specNONE))
     {
       /* Here is where we produce a diagnostic about a reference to a
-         disabled or unimplemented intrinsic, if the diagnostic is desired.  */
+	 disabled or unimplemented intrinsic, if the diagnostic is desired.  */
 
       if ((disabled || unimpl)
 	  && (t != NULL))
@@ -1842,7 +2091,7 @@ ffeintrin_is_intrinsic (char *name, ffelexToken t, bool explicit,
       kind = FFEINFO_kindNONE;
 
       for (i = 0;
-	   (i < ARRAY_SIZE (ffeintrin_gens_[gen].specs))
+	   (((size_t) i) < ARRAY_SIZE (ffeintrin_gens_[gen].specs))
 	   && ((tspec
 		= ffeintrin_gens_[gen].specs[i]) != FFEINTRIN_specNONE);
 	   ++i)
@@ -1965,6 +2214,10 @@ ffeintrin_state_family (ffeintrinFamily family)
       state = ffe_intrinsic_state_f2c ();
       return state;
 
+    case FFEINTRIN_familyF2U:
+      state = ffe_intrinsic_state_unix ();
+      return state;
+
     default:
       assert ("bad family" == NULL);
       return FFE_intrinsicstateDELETED;
