@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
       None
@@ -75,6 +76,7 @@ static char *malloc_types_[]
 /* Static functions (internal). */
 
 static void malloc_kill_area_ (mallocPool pool, mallocArea_ a);
+static void malloc_verify_area_ (mallocPool pool, mallocArea_ a);
 
 /* Internal macros. */
 
@@ -102,6 +104,20 @@ malloc_kill_area_ (mallocPool pool, mallocArea_ a)
 		+ strlen (a->name) + 1);
 }
 
+/* malloc_verify_area_ -- Verify storage area and its object
+
+   malloc_verify_area_(mallocPool pool,mallocArea_ area);
+
+   Does the actual verifying of a storage area.  */
+
+static void
+malloc_verify_area_ (mallocPool pool UNUSED, mallocArea_ a)
+{
+  mallocSize s = a->size;
+
+  assert (strcmp (a->name, ((char *) (a->where)) + s) == 0);
+}
+
 /* malloc_init -- Initialize malloc cluster
 
    malloc_init();
@@ -205,7 +221,8 @@ malloc_pool_kill (mallocPool p)
    Makes a new pool with the given name and default new-chunk allocation.  */
 
 mallocPool
-malloc_pool_new (char *name, mallocPool parent, unsigned long chunks)
+malloc_pool_new (char *name, mallocPool parent,
+		 unsigned long chunks UNUSED)
 {
   mallocPool p;
 
@@ -471,3 +488,27 @@ malloc_resize_ (void *ptr, mallocSize s)
     }
   return ptr;
 }
+
+/* malloc_verify_inpool_ -- Verify object
+
+   Find the mallocArea_ for the pointer, make sure the type is proper, and
+   verify both of them.  */
+
+void
+malloc_verify_inpool_ (mallocPool pool, mallocType_ type, void *ptr,
+		       mallocSize s)
+{
+  mallocArea_ a;
+
+  if (pool == NULL)
+    pool = malloc_pool_image ();
+
+  assert ((pool == malloc_pool_image ())
+	  || malloc_pool_find_ (pool, malloc_pool_image ()));
+
+  a = malloc_find_inpool_ (pool, ptr);
+  assert (a->type == type);
+  if ((type != MALLOC_typeUS_) && (type != MALLOC_typeUSR_))
+    assert (a->size == s);
+  malloc_verify_area_ (pool, a);
+}
