@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.	 */
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.	 */
 
 #include "proj.h"
 #include <ctype.h>
@@ -34,13 +35,13 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.	 */
 #if FFECOM_targetCURRENT == FFECOM_targetFFE
 void
 main (int argc, char *argv[])
-#else
-#if FFECOM_targetCURRENT == FFECOM_targetGCC
+#elif FFECOM_targetCURRENT == FFECOM_targetGCC
 FILE *finput;
 
 int
 yyparse ()
-#endif
+#else
+#error
 #endif
 {
   ffewhereFile wf;
@@ -59,31 +60,33 @@ yyparse ()
       if (!ffe_decode_option (argv[0]))
 	fprintf (stderr, "Unrecognized option: %s\n", argv[0]);
     }
-#else
-#if FFECOM_targetCURRENT == FFECOM_targetGCC
+#elif FFECOM_targetCURRENT == FFECOM_targetGCC
   ffe_set_is_pedantic (pedantic);
-#endif
+#else
+#error
 #endif
 
 #if FFECOM_targetCURRENT == FFECOM_targetFFE
   wf = ffewhere_file_new (NAME_OF_STDIN, strlen (NAME_OF_STDIN));
   ffecom_file (NAME_OF_STDIN);
   ffe_file (wf, stdin);
-#else
-#if FFECOM_targetCURRENT == FFECOM_targetGCC
+#elif FFECOM_targetCURRENT == FFECOM_targetGCC
   wf = ffewhere_file_new (main_input_filename, strlen (main_input_filename));
   ffecom_file (main_input_filename);
   ffe_file (wf, finput);
-#endif
+#else
+#error
 #endif
 
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
   ffecom_finish_compile ();
 
   return 0;
-#else
+#elif FFECOM_targetCURRENT == FFECOM_targetFFE
   ffe_terminate_0 ();
 
   exit (0);
+#else
+#error
 #endif
 }
