@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
       None
@@ -315,7 +316,7 @@ ffesta_save_ (ffelexToken t)
     {
 
       /* If this is an EOS or SEMICOLON token, switch to next handler, else
-         return self as next handler for lexer. */
+	 return self as next handler for lexer. */
 
       switch (ffelex_token_type (t))
 	{
@@ -830,10 +831,10 @@ ffesta_second_ (ffelexToken t)
       break;
 
       /* WARNING: don't put anything that might cause an item to precede
-         FORMAT in the list of possible statements (it's added below) without
-         making sure FORMAT still is first.  It has to run with
-         ffelex_set_names_pure(TRUE), to make sure the lexer delivers NAMES
-         tokens. */
+	 FORMAT in the list of possible statements (it's added below) without
+	 making sure FORMAT still is first.  It has to run with
+	 ffelex_set_names_pure(TRUE), to make sure the lexer delivers NAMES
+	 tokens. */
 
     case FFESTR_firstFORMAT:
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_R1001);
@@ -972,7 +973,7 @@ ffesta_second_ (ffelexToken t)
     default:
 
       /* For now, a decent error message for an unconfirmed stmt, rather than
-         just whatever is at the top of the list. */
+	 just whatever is at the top of the list. */
 
       ffesta_add_possible_nonexec_ ((ffelexHandler) ffestb_unimplemented);
       break;
@@ -1151,7 +1152,7 @@ ffesta_second_ (ffelexToken t)
       else			/* Can't do exec stmt yet. */
 	goto no_stmts;		/* :::::::::::::::::::: */
       /* 1 exec stmt only, but not valid in context, so pretend as though
-         statement is unrecognized. */
+	 statement is unrecognized. */
       break;
 
     default:			/* More than one, so try them in order. */
