@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
       None
@@ -422,7 +423,7 @@ ffetarget_gt_character1 (bool *res, ffetargetCharacter1 l,
    data type info and the number of elements an array (1 for a scalar).	 */
 
 void
-ffetarget_layout (char *error_text, ffetargetAlign *alignment,
+ffetarget_layout (char *error_text UNUSED, ffetargetAlign *alignment,
 		  ffetargetAlign *modulo, ffetargetOffset *size,
 		  ffeinfoBasictype bt, ffeinfoKindtype kt,
 		  ffetargetCharacterSize charsize,
@@ -521,11 +522,11 @@ ffebad
 ffetarget_convert_any_character1_ (char *res, size_t size,
 				   ffetargetCharacter1 l)
 {
-  if (size <= l.length)
+  if (size <= (size_t) l.length)
     {
       char *p;
       ffetargetCharacterSize i;
-      
+
       memcpy (res, l.text, size);
       for (p = &l.text[0] + size, i = l.length - size;
 	   i > 0;
@@ -546,11 +547,11 @@ ffebad
 ffetarget_convert_any_hollerith_ (char *res, size_t size,
 				  ffetargetHollerith l)
 {
-  if (size <= l.length)
+  if (size <= (size_t) l.length)
     {
       char *p;
       ffetargetCharacterSize i;
-      
+
       memcpy (res, l.text, size);
       for (p = &l.text[0] + size, i = l.length - size;
 	   i > 0;
@@ -576,7 +577,7 @@ ffetarget_convert_any_typeless_ (char *res, size_t size,
   unsigned int l3;
   unsigned short int l4;
   unsigned char l5;
-  int size_of;
+  size_t size_of;
   char *p;
 
   if (size >= sizeof (l1))
@@ -710,42 +711,42 @@ ffetarget_convert_character1_integer1 (ffetargetCharacter1 *res,
 				       ffetargetCharacterSize size,
 				       ffetargetInteger1 l, mallocPool pool)
 {
-  unsigned long long int l1;
-  unsigned long int l2;
-  unsigned int l3;
-  unsigned short int l4;
-  unsigned char l5;
-  int size_of;
+  long long int l1;
+  long int l2;
+  int l3;
+  short int l4;
+  char l5;
+  size_t size_of;
   char *p;
 
-  if (size >= sizeof (l1))
+  if (((size_t) size) >= sizeof (l1))
     {
       l1 = l;
       p = (char *) &l1;
       size_of = sizeof (l1);
     }
-  else if (size >= sizeof (l2))
+  else if (((size_t) size) >= sizeof (l2))
     {
       l2 = l;
       p = (char *) &l2;
       size_of = sizeof (l2);
       l1 = l2;
     }
-  else if (size >= sizeof (l3))
+  else if (((size_t) size) >= sizeof (l3))
     {
       l3 = l;
       p = (char *) &l3;
       size_of = sizeof (l3);
       l1 = l3;
     }
-  else if (size >= sizeof (l4))
+  else if (((size_t) size) >= sizeof (l4))
     {
       l4 = l;
       p = (char *) &l4;
       size_of = sizeof (l4);
       l1 = l4;
     }
-  else if (size >= sizeof (l5))
+  else if (((size_t) size) >= sizeof (l5))
     {
       l5 = l;
       p = (char *) &l5;
@@ -764,7 +765,93 @@ ffetarget_convert_character1_integer1 (ffetargetCharacter1 *res,
   else
     {
       res->text = malloc_new_kp (pool, "FFETARGET cvt char1", size);
-      if (size <= size_of)
+      if (((size_t) size) <= size_of)
+	{
+	  int i = size_of - size;
+
+	  memcpy (res->text, p + i, size);
+	  for (; i > 0; ++p, --i)
+	    if (*p != 0)
+	      return FFEBAD_TRUNCATING_NUMERIC;
+	}
+      else
+	{
+	  int i = size - size_of;
+
+	  memset (res->text, 0, i);
+	  memcpy (res->text + i, p, size_of);
+	}
+    }
+
+  if (l1 != l)
+    return FFEBAD_TRUNCATING_NUMERIC;
+  return FFEBAD;
+}
+
+#endif
+/* ffetarget_convert_character1_logical1 -- Raw conversion.  */
+
+#if FFETARGET_okCHARACTER1
+ffebad
+ffetarget_convert_character1_logical1 (ffetargetCharacter1 *res,
+				       ffetargetCharacterSize size,
+				       ffetargetLogical1 l, mallocPool pool)
+{
+  long long int l1;
+  long int l2;
+  int l3;
+  short int l4;
+  char l5;
+  size_t size_of;
+  char *p;
+
+  if (((size_t) size) >= sizeof (l1))
+    {
+      l1 = l;
+      p = (char *) &l1;
+      size_of = sizeof (l1);
+    }
+  else if (((size_t) size) >= sizeof (l2))
+    {
+      l2 = l;
+      p = (char *) &l2;
+      size_of = sizeof (l2);
+      l1 = l2;
+    }
+  else if (((size_t) size) >= sizeof (l3))
+    {
+      l3 = l;
+      p = (char *) &l3;
+      size_of = sizeof (l3);
+      l1 = l3;
+    }
+  else if (((size_t) size) >= sizeof (l4))
+    {
+      l4 = l;
+      p = (char *) &l4;
+      size_of = sizeof (l4);
+      l1 = l4;
+    }
+  else if (((size_t) size) >= sizeof (l5))
+    {
+      l5 = l;
+      p = (char *) &l5;
+      size_of = sizeof (l5);
+      l1 = l5;
+    }
+  else
+    {
+      assert ("stumped by conversion from logical1!" == NULL);
+      abort ();
+    }
+
+  res->length = size;
+  if (size == 0)
+    res->text = NULL;
+  else
+    {
+      res->text = malloc_new_kp (pool, "FFETARGET cvt char1", size);
+      if (((size_t) size) <= size_of)
 	{
 	  int i = size_of - size;
 
@@ -801,37 +888,37 @@ ffetarget_convert_character1_typeless (ffetargetCharacter1 *res,
   unsigned int l3;
   unsigned short int l4;
   unsigned char l5;
-  int size_of;
+  size_t size_of;
   char *p;
 
-  if (size >= sizeof (l1))
+  if (((size_t) size) >= sizeof (l1))
     {
       l1 = l;
       p = (char *) &l1;
       size_of = sizeof (l1);
     }
-  else if (size >= sizeof (l2))
+  else if (((size_t) size) >= sizeof (l2))
     {
       l2 = l;
       p = (char *) &l2;
       size_of = sizeof (l2);
       l1 = l2;
     }
-  else if (size >= sizeof (l3))
+  else if (((size_t) size) >= sizeof (l3))
     {
       l3 = l;
       p = (char *) &l3;
       size_of = sizeof (l3);
       l1 = l3;
     }
-  else if (size >= sizeof (l4))
+  else if (((size_t) size) >= sizeof (l4))
     {
       l4 = l;
       p = (char *) &l4;
       size_of = sizeof (l4);
       l1 = l4;
     }
-  else if (size >= sizeof (l5))
+  else if (((size_t) size) >= sizeof (l5))
     {
       l5 = l;
       p = (char *) &l5;
@@ -850,7 +937,7 @@ ffetarget_convert_character1_typeless (ffetargetCharacter1 *res,
   else
     {
       res->text = malloc_new_kp (pool, "FFETARGET cvt char1", size);
-      if (size <= size_of)
+      if (((size_t) size) <= size_of)
 	{
 	  int i = size_of - size;
 
@@ -2121,6 +2208,13 @@ ffetarget_typeless_hex (ffetargetTypeless *xvalue, ffelexToken token)
   return !bad_digit && !overflow;
 }
 
+void
+ffetarget_verify_character1 (mallocPool pool, ffetargetCharacter1 val)
+{
+  if (val.length != 0)
+    malloc_verify_kp (pool, val.text, val.length);
+}
+
 /* This is like memcpy.	 It is needed because some systems' header files
    don't declare memcpy as a function but instead
    "#define memcpy(to,from,len) something".  */
