@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Owning Modules:
       target.c
@@ -117,29 +118,40 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifndef FFETARGET_defaultSTATE_MIL
 #define FFETARGET_defaultSTATE_MIL FFE_intrinsicstateENABLED
 #endif
+#ifndef FFETARGET_defaultSTATE_UNIX
+#define FFETARGET_defaultSTATE_UNIX FFE_intrinsicstateENABLED
+#endif
 #ifndef FFETARGET_defaultSTATE_VXT
 #define FFETARGET_defaultSTATE_VXT FFE_intrinsicstateDELETED
 #endif
 
+#ifndef FFETARGET_defaultFIXED_LINE_LENGTH
+#define FFETARGET_defaultFIXED_LINE_LENGTH 72
+#endif
+
 /* 1 if external Fortran names ("FOO" in SUBROUTINE FOO, COMMON /FOO/,
    and even enforced/default-for-unnamed PROGRAM, blank-COMMON, and
    BLOCK DATA names, but not names of library functions implementing
    intrinsics or names of local/internal variables) should have an
    underscore appended (for compatibility with existing systems).  */
 
-#ifndef FFETARGET_isEXTERNAL_UNDERSCORED
-#define FFETARGET_isEXTERNAL_UNDERSCORED 1
+#ifndef FFETARGET_defaultEXTERNAL_UNDERSCORED
+#define FFETARGET_defaultEXTERNAL_UNDERSCORED 1
 #endif
 
 /* 1 if external Fortran names with underscores already in them should
    have an extra underscore appended (in addition to the one they
-   might already have appened if FFETARGET_isEXTERNAL_UNDERSCORED).  */
+   might already have appened if FFETARGET_defaultEXTERNAL_UNDERSCORED).
+   NOTE: See gcc/f/com.c for other changes that need to be made
+   if this is not #define'd to be 1, but underscoring is turned on.
+   Right now there's only the option to disable underscoring in general,
+   not to change just this setting.  */
 
-#ifndef FFETARGET_isUNDERSCORED_EXTERNAL_UNDERSCORED
-#define FFETARGET_isUNDERSCORED_EXTERNAL_UNDERSCORED FFETARGET_isEXTERNAL_UNDERSCORED
+#ifndef FFETARGET_defaultUNDERSCORED_EXTERNAL_UNDERSCORED
+#define FFETARGET_defaultUNDERSCORED_EXTERNAL_UNDERSCORED 1
 #endif
 
-/* If FFETARGET_isEXTERNAL_UNDERSCORED is 0, the following definitions
+/* If FFETARGET_defaultEXTERNAL_UNDERSCORED is 0, the following definitions
    might also need to be overridden to make g77 objects compatible with
    f2c+gcc objects.  Although I don't think the unnamed BLOCK DATA one
    is an issue at all.  Of course, on some systems it isn't f2c
@@ -220,9 +232,12 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #define FFETARGET_okREAL1 1
 #define FFETARGET_okREAL2 1
 #define FFETARGET_okREAL3 0
+#define FFETARGET_okREALQUAD FFETARGET_okREAL3
 #define FFETARGET_okCOMPLEX1 1
 #define FFETARGET_okCOMPLEX2 1
 #define FFETARGET_okCOMPLEX3 0
+#define FFETARGET_okCOMPLEXDOUBLE FFETARGET_okCOMPLEX2
+#define FFETARGET_okCOMPLEXQUAD FFETARGET_okCOMPLEX3
 #define FFETARGET_okCHARACTER1 1
 
 #define FFETARGET_f2cTYUNKNOWN 0
@@ -245,7 +260,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 typedef unsigned char ffetargetAlign;	/* ffetargetOffset for alignment. */
 #define ffetargetAlign_f ""
-typedef unsigned long ffetargetCharacterSize;
+typedef long ffetargetCharacterSize;
 #define ffetargetCharacterSize_f "l"
 typedef void (*ffetargetCopyfunc) (void *, void *, size_t);
 typedef ffetargetCharacterSize ffetargetHollerithSize;
@@ -580,6 +595,10 @@ ffebad ffetarget_convert_character1_integer1 (ffetargetCharacter1 *res,
 					      ffetargetCharacterSize res_size,
 					      ffetargetInteger1 l,
 					      mallocPool pool);
+ffebad ffetarget_convert_character1_logical1 (ffetargetCharacter1 *res,
+					      ffetargetCharacterSize res_size,
+					      ffetargetLogical1 l,
+					      mallocPool pool);
 ffebad ffetarget_convert_character1_typeless (ffetargetCharacter1 *res,
 					    ffetargetCharacterSize res_size,
 					      ffetargetTypeless l,
@@ -797,6 +816,7 @@ bool ffetarget_real8 (ffetargetReal8 *value, ffelexToken integer,
 bool ffetarget_typeless_binary (ffetargetTypeless *value, ffelexToken token);
 bool ffetarget_typeless_octal (ffetargetTypeless *value, ffelexToken token);
 bool ffetarget_typeless_hex (ffetargetTypeless *value, ffelexToken token);
+void ffetarget_verify_character1 (mallocPool pool, ffetargetCharacter1 val);
 int ffetarget_num_digits_ (ffelexToken t);
 void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 
@@ -1264,7 +1284,7 @@ void *ffetarget_memcpy_ (void *dst, void *src, size_t len);
 #define ffetarget_init_3()
 #define ffetarget_init_4()
 #define ffetarget_integerdefault_is_magical(i) \
-  (i == FFETARGET_integerBIG_MAGICAL)
+  (((unsigned long int) i) == FFETARGET_integerBIG_MAGICAL)
 #ifdef REAL_ARITHMETIC
 #define ffetarget_iszero_real1(l) \
   ({ REAL_VALUE_TYPE lr; \
