@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
       None.
@@ -29,6 +30,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 /* Include files. */
 
+#include <ctype.h>
 #include "proj.h"
 #include "top.h"
 #include "bad.h"
@@ -53,7 +55,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "target.h"
 #include "where.h"
 #if FFECOM_targetCURRENT == FFECOM_targetGCC
-#include "flags.h"
+#include "flags.j"
 #endif
 
 /* Externals defined here. */
@@ -62,11 +64,14 @@ int flag_traditional;		/* Shouldn't need this (C front end only)! */
 bool ffe_is_90_ = FFETARGET_defaultIS_90;
 bool ffe_is_automatic_ = FFETARGET_defaultIS_AUTOMATIC;
 bool ffe_is_backslash_ = FFETARGET_defaultIS_BACKSLASH;
+bool ffe_is_underscoring_ = FFETARGET_defaultEXTERNAL_UNDERSCORED
+  || FFETARGET_defaultUNDERSCORED_EXTERNAL_UNDERSCORED;
 bool ffe_is_dollar_ok_ = FFETARGET_defaultIS_DOLLAR_OK;
 bool ffe_is_f2c_ = FFETARGET_defaultIS_F2C;
 bool ffe_is_f2c_library_ = FFETARGET_defaultIS_F2C_LIBRARY;
 bool ffe_is_ffedebug_ = FALSE;
 bool ffe_is_free_form_ = FFETARGET_defaultIS_FREE_FORM;
+bool ffe_is_ident_ = TRUE;
 bool ffe_is_init_local_zero_ = FFETARGET_defaultIS_INIT_LOCAL_ZERO;
 bool ffe_is_mainprog_;		/* TRUE if current prog unit known to be
 				   main. */
@@ -86,7 +91,9 @@ ffeIntrinsicState ffe_intrinsic_state_dcp_ = FFETARGET_defaultSTATE_DCP;
 ffeIntrinsicState ffe_intrinsic_state_f2c_ = FFETARGET_defaultSTATE_F2C;
 ffeIntrinsicState ffe_intrinsic_state_f90_ = FFETARGET_defaultSTATE_F90;
 ffeIntrinsicState ffe_intrinsic_state_mil_ = FFETARGET_defaultSTATE_MIL;
+ffeIntrinsicState ffe_intrinsic_state_unix_ = FFETARGET_defaultSTATE_UNIX;
 ffeIntrinsicState ffe_intrinsic_state_vxt_ = FFETARGET_defaultSTATE_VXT;
+int ffe_fixed_line_length_ = FFETARGET_defaultFIXED_LINE_LENGTH;
 mallocPool ffe_file_pool_ = NULL;
 mallocPool ffe_any_unit_pool_ = NULL;
 mallocPool ffe_program_unit_pool_ = NULL;
@@ -118,9 +125,21 @@ bool ffe_in_4 = FALSE;
 
 /* Static functions (internal). */
 
+static bool ffe_is_digit_string_ (char *s);
 
 /* Internal macros. */
 
+static bool
+ffe_is_digit_string_ (char *s)
+{
+  char *p;
+
+  for (p = s; isdigit (*p); ++p)
+    ;
+
+  return (p != s) && (*p == '\0');
+}
+
 /* Handle command-line options.	 Returns 0 if unrecognized, 1 if
    recognized and handled.  */
 
@@ -133,6 +152,13 @@ ffe_decode_option (char *opt)
     {
       if (strcmp (&opt[2], "version") == 0)
 	ffe_set_is_version (TRUE);
+      else if (strcmp (&opt[2], "null-version") == 0)
+	;			/* Someday generate program to print version
+				   info.  */
+      else if (strcmp (&opt[2], "ident") == 0)
+	ffe_set_is_ident (TRUE);
+      else if (strcmp (&opt[2], "no-ident") == 0)
+	ffe_set_is_ident (FALSE);
       else if (strcmp (&opt[2], "f90") == 0)
 	ffe_set_is_90 (TRUE);
       else if (strcmp (&opt[2], "no-f90") == 0)
@@ -201,6 +227,10 @@ ffe_decode_option (char *opt)
 	ffe_set_is_backslash (TRUE);
       else if (strcmp (&opt[2], "no-backslash") == 0)
 	ffe_set_is_backslash (FALSE);
+      else if (strcmp (&opt[2], "underscoring") == 0)
+	ffe_set_is_underscoring (TRUE);
+      else if (strcmp (&opt[2], "no-underscoring") == 0)
+	ffe_set_is_underscoring (FALSE);
       else if (strcmp (&opt[2], "intrin-case-initcap") == 0)
 	ffe_set_case_intrin (FFE_caseINITCAP);
       else if (strcmp (&opt[2], "intrin-case-upper") == 0)
@@ -305,6 +335,14 @@ ffe_decode_option (char *opt)
 	ffe_set_intrinsic_state_mil (FFE_intrinsicstateDISABLED);
       else if (strcmp (&opt[2], "mil-intrinsics-enable") == 0)
 	ffe_set_intrinsic_state_mil (FFE_intrinsicstateENABLED);
+      else if (strcmp (&opt[2], "unix-intrinsics-delete") == 0)
+	ffe_set_intrinsic_state_unix (FFE_intrinsicstateDELETED);
+      else if (strcmp (&opt[2], "unix-intrinsics-hide") == 0)
+	ffe_set_intrinsic_state_unix (FFE_intrinsicstateHIDDEN);
+      else if (strcmp (&opt[2], "unix-intrinsics-disable") == 0)
+	ffe_set_intrinsic_state_unix (FFE_intrinsicstateDISABLED);
+      else if (strcmp (&opt[2], "unix-intrinsics-enable") == 0)
+	ffe_set_intrinsic_state_unix (FFE_intrinsicstateENABLED);
       else if (strcmp (&opt[2], "vxt-intrinsics-delete") == 0)
 	ffe_set_intrinsic_state_vxt (FFE_intrinsicstateDELETED);
       else if (strcmp (&opt[2], "vxt-intrinsics-hide") == 0)
@@ -313,6 +351,18 @@ ffe_decode_option (char *opt)
 	ffe_set_intrinsic_state_vxt (FFE_intrinsicstateDISABLED);
       else if (strcmp (&opt[2], "vxt-intrinsics-enable") == 0)
 	ffe_set_intrinsic_state_vxt (FFE_intrinsicstateENABLED);
+      else if (strncmp (&opt[2], "fixed-line-length-",
+			strlen ("fixed-line-length-")) == 0)
+	{
+	  char *len = &opt[2] + strlen ("fixed-line-length-");
+
+	  if (strcmp (len, "none") == 0)
+	    ffe_set_fixed_line_length (0);
+	  else if (ffe_is_digit_string_ (len))
+	    ffe_set_fixed_line_length (atol (len));
+	  else
+	    return 0;
+	}
       else
 	return 0;
     }
@@ -340,7 +390,6 @@ ffe_decode_option (char *opt)
 	ffe_set_is_warn_implicit(FALSE);
       else if (!strcmp (&opt[2], "all"))
 	{
-	  extra_warnings = 1;
 	  /* We save the value of warn_uninitialized, since if they put
 	     -Wuninitialized on the command line, we need to generate a
 	     warning about not using it without also specifying -O.  */
@@ -370,12 +419,11 @@ ffe_file (ffewhereFile wf, FILE *f)
 {
   ffe_init_1 ();
   ffelex_set_handler ((ffelexHandler) ffest_first);
-  ffewhere_file_begin (NULL, wf);
+  ffewhere_file_set (wf, TRUE, 0);
   if (ffe_is_free_form_)
     ffelex_file_free (wf, f);
   else
     ffelex_file_fixed (wf, f);
-  ffewhere_file_end (wf, NULL);
   ffest_eof ();
   ffe_terminate_1 ();
 }
