@@ -16,7 +16,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Fortran; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
+02111-1307, USA.
 
    Related Modules:
       None
@@ -28,33 +29,20 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
    Modifications:
 */
 
-/* Include files. */
-
 #include "proj.h"
 #include "zzz.h"
 
-/* Externals defined here. */
+/* If you want to override the version date/time info with your own
+   macros, e.g. for a consistent distribution when bootstrapping,
+   go ahead!  */
 
-char *ffezzz_version_string = "0.5.14";
-char *ffezzz_date = __DATE__;
-char *ffezzz_time = __TIME__;
+#ifndef FFEZZZ_DATE
+#define FFEZZZ_DATE __DATE__
+#endif
+#ifndef FFEZZZ_TIME
+#define FFEZZZ_TIME __TIME__
+#endif
 
-/* Simple definitions and enumerations. */
-
-
-/* Internal typedefs. */
-
-
-/* Private include files. */
-
-
-/* Internal structure definitions. */
-
-
-/* Static objects accessed by functions in this module.	 */
-
-
-/* Static functions (internal). */
-
-
-/* Internal macros. */
+char *ffezzz_version_string = "0.5.16";
+char *ffezzz_date = FFEZZZ_DATE;
+char *ffezzz_time = FFEZZZ_TIME;
